// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2020 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// ====================================================================================
// 
//  __      __      __
//  \ \    /  \    / /   [File name   ] lvds_video_top.v
//   \ \  / /\ \  / /    [Description ] LVDS Video
//    \ \/ /  \ \/ /     [Timestamp   ] Friday November 20 14:00:30 2020
//     \  /    \  /      [version     ] 1.0
//      \/      \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | Caojie     | 11/20/20     | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========

module lvds_video_top
(
    input          I_clk       ,  //50MHz      
    input          I_rst_n     ,
    output [3:0]   O_led       ,    
    output         O_clkout_p  ,
    output         O_clkout_n  ,
    output [3:0]   O_dout_p    ,
    output [3:0]   O_dout_n    
);

//======================================================
reg  [31:0] run_cnt;
wire        running;

//-----------
wire        ch0_de_in;
wire        ch0_hs_in;
wire        ch0_vs_in;
wire[7:0]   ch0_data_r;
wire[7:0]   ch0_data_g;
wire[7:0]   ch0_data_b;    

reg         vs_r;
reg  [9:0]  cnt_vs;

//-------------------  
wire        serial_clk;
wire        pll_lock;

wire        pixel_clk;

wire        rst_n;

//===================================================
//LED test
always @(posedge I_clk or negedge I_rst_n)//I_clk
begin
    if(!I_rst_n)
        run_cnt <= 32'd0;
    else if(run_cnt >= 32'd50_000_000)
        run_cnt <= 32'd0;
    else
        run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;

assign  O_led[0] = running;
assign  O_led[1] = 1'b0;
assign  O_led[2] = 1'b1;
assign  O_led[3] = 1'b0;

//=============================================================================
//testpattern
testpattern testpattern_inst
(
    .I_pxl_clk   (pixel_clk         ),//pixel clock 
    .I_rst_n     (rst_n             ),//low active 
    .I_mode      ({1'b0,cnt_vs[9:8]}),//data select
    .I_single_r  (8'd0              ),
    .I_single_g  (8'd0              ),                  //74.25MHz   //83.5MHz    
    .I_single_b  (8'd255            ),                  //1280x720   //1280x800   
    .I_h_total   (12'd1680          ),//hor total time  // 12'd1650  // 12'd1680  
    .I_h_sync    (12'd128           ),//hor sync time   // 12'd40    // 12'd128   
    .I_h_bporch  (12'd200           ),//hor back porch  // 12'd220   // 12'd200   
    .I_h_res     (12'd1280          ),//hor resolution  // 12'd1280  // 12'd1280  
    .I_v_total   (12'd831           ),//ver total time  // 12'd750   // 12'd831   
    .I_v_sync    (12'd6             ),//ver sync time   // 12'd5     // 12'd6     
    .I_v_bporch  (12'd22            ),//ver back porch  // 12'd20    // 12'd22    
    .I_v_res     (12'd800           ),//ver resolution  // 12'd720   // 12'd800   
    .I_hs_pol    (1'b1              ),
    .I_vs_pol    (1'b1              ),
    .O_de        (ch0_de_in         ),   
    .O_hs        (ch0_hs_in         ),
    .O_vs        (ch0_vs_in         ),
    .O_data_r    (ch0_data_r        ),   
    .O_data_g    (ch0_data_g        ),
    .O_data_b    (ch0_data_b        )
);

always@(posedge pixel_clk)
begin
    vs_r<=ch0_vs_in;
end

always@(posedge pixel_clk or negedge rst_n)
begin
    if(!rst_n)
        cnt_vs<=10'd0;
    else if(vs_r && !ch0_vs_in) //vs24 falling edge
        cnt_vs<=cnt_vs+1'b1;
end 

//===================================================================================
//LVDS TX
Gowin_rPLL Gowin_rPLL_inst
(
    .clkout(serial_clk), //x3.5
    .lock  (pll_lock  ), //
    .clkin (I_clk     )  //
);

assign rst_n = I_rst_n & pll_lock;

CLKDIV CLKDIV_inst
(
    .RESETN(rst_n     ),
    .HCLKIN(serial_clk), //x3.5
    .CALIB (1'b1      ),
    .CLKOUT(pixel_clk )  //x1
);
defparam CLKDIV_inst.DIV_MODE = "3.5" ;

LVDS_7to1_TX_Top LVDS_7to1_TX_Top_inst
(
    .I_rst_n       (rst_n       ),
    .I_serial_clk  (serial_clk  ), //x3.5
    .I_pix_clk     (pixel_clk   ), //x1                       
    .I_vs          (ch0_vs_in   ), 
    .I_hs          (ch0_hs_in   ),
    .I_de          (ch0_de_in   ),
    .I_data_r      (ch0_data_r  ),
    .I_data_g      (ch0_data_g  ),
    .I_data_b      (ch0_data_b  ), 
    .O_clkout_p    (O_clkout_p  ), 
    .O_clkout_n    (O_clkout_n  ),
    .O_dout_p      (O_dout_p    ),    
    .O_dout_n      (O_dout_n    ) 
);

endmodule